load('api_config.js');
load('api_events.js');
load('api_gpio.js');
load('api_log.js');
load('api_net.js');
load('api_rpc.js');
load('api_sys.js');
load('api_timer.js');
load('api_uart.js');
load('api_m5stack.js');

let devID = Cfg.get('device.id');
let greeting = '';
let btnc = [-1, 0, 0, 0];
let netStatus = null;


M5stack.displayInit();
M5stack.printCentered(M5stack.screenWidth / 2, M5stack.line(0), devID);
let strip = M5stack.pixelInit();
         // pixel, R,  G,  B);
strip.setPixel(0, 50,  0,  0); // top right
strip.setPixel(2,  0, 50,  0);
strip.setPixel(4,  0,  0, 50);
strip.setPixel(M5stack.numPixels-1, 25, 25, 25); // top left
strip.show();
//strip.clear();
//strip.free();


function printNetStatus() {
  if (!netStatus) netStatus = 'not configured';
  M5stack.setFont(1);
  M5stack.setFgColor(M5stack.BLUE);
  M5stack.print(5, M5stack.line(1), 'WiFi: ' + netStatus + '         ');
}

function printTime() {
  M5stack.setFont(1);
  M5stack.setFgColor(M5stack.RED);
  let now = Timer.now();
  let ts = Timer.fmt('%H:%M:%S', now);
  M5stack.print(5, M5stack.line(2), 'Time: ' + ((now > 1500000000) ? ts : 'not set') + '   ');
}

function printGreeting() {
  M5stack.setFont(1);
  M5stack.setFgColor(M5stack.GREEN);
  if (greeting) {
    M5stack.printCentered(160, M5stack.line(5), greeting);
  } else {
    M5stack.clearLine(5);
  }
}

function reportBtnPress(n) {
  btnc[n] = btnc[n] + 1;

  printBtnStatus();
}


function printBtnStatus() {
  M5stack.setFont(2);
  M5stack.setFgColor(M5stack.GREEN);
  let y = M5stack.line(-1);
  M5stack.printCentered(65, y, JSON.stringify(btnc[1]))
  M5stack.printCentered(160, y, JSON.stringify(btnc[2]))
  M5stack.printCentered(255, y, JSON.stringify(btnc[3]))
}

function printStatus() {
  printNetStatus();
  printTime();
  printGreeting();
  printBtnStatus();
}

// Monitor network connectivity.
Event.addGroupHandler(Net.EVENT_GRP, function(ev, evdata, arg) {
  if (ev === Net.STATUS_DISCONNECTED) {
    netStatus = 'not connected';
  } else if (ev === Net.STATUS_CONNECTING) {
    netStatus = 'connecting';
  } else if (ev === Net.STATUS_GOT_IP) {
    netStatus = 'connected';
  }
  printNetStatus();
}, null);

GPIO.set_button_handler(M5stack.BTN1, GPIO.PULL_UP, GPIO.INT_EDGE_NEG, 20, function() { reportBtnPress(1) }, null);
GPIO.set_button_handler(M5stack.BTN2, GPIO.PULL_UP, GPIO.INT_EDGE_NEG, 20, function() { reportBtnPress(2) }, null);
GPIO.set_button_handler(M5stack.BTN3, GPIO.PULL_UP, GPIO.INT_EDGE_NEG, 20, function() { reportBtnPress(3) }, null);
RPC.addHandler('M5.SetGreeting', function(args) {
  if (args.greeting === undefined) {
    return {"error": 400, "message": "greeting not specified"};
  }
  M5stack.setFont(0);
  greeting = '';
  printGreeting();
  greeting = args.greeting;
  printGreeting();
});


printStatus();
Timer.set(1000 /* 1 sec */, Timer.REPEAT, printStatus, null);


// Configure UART at 115200 baud
let uartNo = 2;
UART.setConfig(uartNo, {    // manually configure pins on any UART (UART0 is tied to USB and used for logging)
  baudRate: 115200,
  rxLingerMicros: 75,
  esp32: {                  // this is not actually needed if using UART2 as these are default pins
    gpio: {
      rx: 16,
      tx: 17,
    },
  },
});

// Set dispatcher callback, it will be called whenever new Rx data or space in
// the Tx buffer becomes available
UART.setDispatcher(uartNo, function(uartNo) {
  if (UART.readAvail(uartNo) > 0) {
    // Received new data
    let data = UART.read(uartNo);
    let ra = data.length; // account for more data arriving between readAvail() and read()
    Log.print(Log.INFO, 'Received UART data: ' + data);
  }
}, null);

// Enable Rx
UART.setRxEnabled(uartNo, true);

// Send UART data every second
Timer.set(1000 /* milliseconds */, Timer.REPEAT, function() {
  UART.write(uartNo, 'Hello, world!\r\n');
}, null);

